% Jim Wang
% 9/9/08
%
% solves for membrane potential and returns firing rate in (spike/ms)
%
% heas.m
%
% I_app = applied current (A/cm^2)
% dt = time step (ms)
% T = total time of simulation (ms)
% v0 = resting membrane potential (mV)
% tau_A = A time constant (ms)
% gbar_A = gating variable for A (mS/cm^2)
% g_L = gating leaking variable (mS/cm^2)
% enter graph = 1 if want plot of v versus t
% enter graph = 0 if else
%
% Example: heas(1.6,.01,500,-73,20,20,0.05,1)
function spikefreq = heas(I_app,dt,T,v0,tau_A,gbar_A,g_L,graph)
Cm = 1;         % F/cm^2
gbar_na = 100;  % mS/cm^2
gbar_k = 40;    % mS/cm^2
E_na = 55;      % mV
E_k = -80;      % mV
E_L = -65;      % mV
phi = 0.1;

t = 0:dt:T;

% initializing
v = zeros(1,length(t));
h = v;  
n = v;  
b = v;  
m = v;

vthreshold = 30;    % threshold for firing (mV)
v(1) = v0;          % resting membrane potential (mV)

% how to find initial conditions for h, n, and b
for i = 1:length(t)-1
    alpha_m = -0.1*(v(i)+30)/(exp(-0.1*(v(i)+30))-1);
    beta_m = 4*exp(-(v(i)+55)/18);

    alpha_h = 0.07*exp(-(v(i)+44)/20);
    beta_h = 1/(exp(-0.1*(v(i)+14))+1);

    alpha_n = -0.01*(v(i)+34)/(exp(-0.1*(v(i)+34))-1);
    beta_n = 0.125*exp(-(v(i)+44)/80);
    
    a_inf = 1/(exp(-(v(i)+50)/20)+1);
    b_inf = 1/(exp((v(i)+80)/6)+1);
    
    m_inf = alpha_m/(alpha_m+beta_m);
    
    tau_h = phi/(alpha_h+beta_h);
    h_inf = alpha_h/(alpha_h+beta_h);
    
    tau_n = phi/(alpha_n+beta_n);
    n_inf = alpha_n/(alpha_n+beta_n);
    
    if i == 1   % initial values
        h(i) = h_inf;
        n(i) = n_inf;
        b(i) = b_inf;
    end    
    m(i) = m_inf;
    
    h(i+1) = (h(i)*tau_h+dt*h_inf)/(tau_h+dt);   % hybrid euler
    n(i+1) = (n(i)*tau_n+dt*n_inf)/(tau_n+dt);
    b(i+1) = (b(i)*tau_A+dt*b_inf)/(tau_A+dt);
    
    top = v(i) + dt/Cm*(g_L*E_L+gbar_na*m_inf^3*h(i+1)*E_na+...
        gbar_k*n(i+1)^4*E_k+gbar_A*a_inf^3*b(i+1)*E_k+I_app);
    bottom = 1 + dt/Cm*(g_L+...
        gbar_na*m_inf^3*h(i+1)+gbar_k*n(i+1)^4+gbar_A*a_inf^3*b(i+1));
    
    v(i+1) = top/bottom; 
end
if graph
    figure(2)
    plot(t,v)
    set(gca,'FontSize',18)
    xlabel('t')
    ylabel('v')
    title(['I_{app} = ' num2str(I_app) ' A/cm^2'])
    axis([0 T -80 50])
    figure(3)
    set(gca,'FontSize',18)
    plot(t,n,'r',t,h,'b',t,b,'g',t,m,'k')
    legend('n','h','b','m','Location','NorthWest')
end

% determining the firing rate
v2 = diff(v(v>vthreshold));          % difference between adjacent elements in v2
v2negloc = find(v2<0);               % location of negative v2
v2poscross = (v2(v2negloc-1)>0);     % location of positive v2 for which the next element if negative
                                     % i.e. location of change from positive to negative
numspike = sum(v2poscross);          % number of spikes
spikefreq = numspike/T;              % firing rate
